<?php
class field_manager
{
	private $fields;

	public function __construct ($fields)
	{
		lib_event::log (lib_event::L_DEBUG, "Start of fields creation");
		$this->fields = array ();
		foreach ($fields as $name => $info) {
			list ($type, $source, $list_name, $list_initial, $fetch, $checks) = $info;
			switch ($type) {
			case const_mvc::FIELD_AREA:
				$class = "form_field_area";
				break;
			case const_mvc::FIELD_CHECKBOX:
				$class = "form_field_checkbox";
				break;
			case const_mvc::FIELD_FILE:
				$class = "form_field_file";
				break;
			case const_mvc::FIELD_HIDDEN:
				$class = "form_field_hidden";
				break;
			case const_mvc::FIELD_IMAGE:
				$class = "form_field_image";
				break;
			case const_mvc::FIELD_MULTIBOX:
				$class = "form_field_multibox";
				break;
			case const_mvc::FIELD_PASSWORD:
				$class = "form_field_password";
				break;
			case const_mvc::FIELD_SELECT:
				$class = "form_field_select";
				break;
			case const_mvc::FIELD_TEXT:
				$class = "form_field_text";
				break;
			case const_mvc::FIELD_SUBMIT:
				$class = "form_field_submit";
				break;
			default:
				lib_event::fatal ("Unknown field type \"%s\"", $type);
			}
			if ($source !== null) {
				switch ($source) {
				case const_mvc::SOURCE_GET:
					$object = $GLOBALS ["_GET"];
					break;
				case const_mvc::SOURCE_POST:
					$object = $GLOBALS ["_POST"];
					break;
				default:
					lib_event::fatal ("Unknown source \"%s\"", $source);
				}
			} else {
				$object = null;
			}
			$this->fields [$name] = new $class (array (
				"name" => $name,
				"type" => $type,
				"source" => $object,
				"list_name" => $list_name,
				"list_initial" => $list_initial,
				"fetch" => $fetch,
				"checks" => $checks
			));
		}
		lib_event::log (lib_event::L_DEBUG, "End of fields creation");
	}

	public function field_exists ($name)
	{
		return (array_key_exists ($name, $this->fields));
	}

	public function field_build_html ($name, $attribs)
	{
		return ($this->fields [$name]->build_html ($attribs));
	}

	public function field_build_error ($name)
	{
		return ($this->fields [$name]->build_error ());
	}

	public function array_get ()
	{
		$result = array ();
		foreach ($this->fields as $name => $one_field) {
			$result [$name] = $one_field->array_get ();
		}
		return ($result);
	}

	public function array_set ($fields)
	{
		foreach ($this->fields as $name => $one_field) {
			$one_field->array_set ($fields [$name]);
		}
	}

	static public function is_ok ($fields)
	{
		foreach ($fields as $one_field) {
			if ($one_field ["error"] !== null) {
				return (false);
			}
		}
		return (true);
	}
}

interface form_field
{
	static public function get_capabilities ();
	public function build_html ($attribs);
	public function build_error ();
	public function array_get ();
	public function array_set ($field);
}

abstract class form_field_parent implements form_field
{
	const ERR_NOT_AVAILABLE = "This field is not available";

	protected $name;
	protected $list_name;
	protected $list_initial;
	protected $value;
	protected $error;

	public function __construct ($info)
	{
		$this->name = $info ["name"];
		$this->list_name = $info ["list_name"];
		$this->list_initial = $info ["list_initial"];
		if ($info ["fetch"] == false) {
			$this->value = $this->_int_get_empty ();
			$this->error = null;
		} else if (($value = $this->_int_fetch ($info ["source"])) === null) {
			$this->value = $this->_int_get_empty ();
			$this->error = self::ERR_NOT_AVAILABLE;
		} else if (($error = $this->_int_check ($value, $info ["checks"])) != null) {
			$this->value = $value;		// FIXME: Allow some way to specify whether values that fail checks will be retained or replaced by empty values
			$this->error = $error;
		} else {
			$this->value = $value;
			$this->error = null;
		}
		lib_event::log (
			lib_event::L_DEBUG,
			"Created field \"%s\": \"%s\"",
			$info ["name"],
			$this->error
		);
	}

	private function _int_check ($value, $checks)
	{
		foreach ($checks as $one_check) {
			list ($type, $message, $text, $int_1, $int_2) = $one_check;
			switch ($type) {
			case const_mvc::CHECK_INTEGER:
				$error = field_check::check_integer ($value, $int_1, $int_2);
				break;
			case const_mvc::CHECK_LENGTH:
				$error = field_check::check_length ($value, $int_1, $int_2);
				break;
			case const_mvc::CHECK_LIST:
				$error = field_check::check_list (
					$value,
					$this->list_name,
					$this->list_initial !== null
				);
				break;
			case const_mvc::CHECK_PCRE:
				$error = field_check::check_pcre ($value, $text);
				break;
			case const_mvc::CHECK_UPLOAD:
				$error = field_check::check_upload ($value);
				break;
			default:
				lib_event::fatal ("Unknown check type \"%s\"", $type);
			}
			if ($error !== null) {
				return (($message !== null) ? $message : $error);
			}
		}
		return (null);
	}

	abstract protected function _int_get_empty ();
	abstract protected function _int_fetch ($source);	/* Return null if fetch has failed */

	public function build_error ()
	{
		return (
			($this->error !== null) ?
			htmlspecialchars ($this->error) :
			""
		);
	}

	public function array_get ()
	{
		return (array (
			"value" => $this->value,
			"error" => $this->error
		));
	}

	public function array_set ($field)
	{
		$this->value = $field ["value"];
		$this->error = $field ["error"];
	}
}

class form_field_area extends form_field_parent
{
	protected function _int_get_empty ()
	{
		return ("");
	}

	protected function _int_fetch ($source)
	{
		if (array_key_exists ($this->name, $source) == false) {
			return (null);
		}
		return ($source [$this->name]);
	}

	static public function get_capabilities ()
	{
		return (array (
			"source" => true,
			"list_name" => false,
			"list_initial" => false
		));
	}

	public function build_html ($attribs)
	{
		return (
			"<textarea" .
				" name=\"" . $this->name . "\"" .
				(($attribs != "") ? " " . $attribs : "") .
			">" .
				htmlspecialchars ($this->value) .
			"</textarea>"
		);
	}
}

class form_field_checkbox extends form_field_parent
{
	protected function _int_get_empty ()
	{
		return (false);
	}

	protected function _int_fetch ($source)
	{
		return (array_key_exists ($this->name, $source));
	}

	static public function get_capabilities ()
	{
		return (array (
			"source" => true,
			"list_name" => false,
			"list_initial" => false
		));
	}

	public function build_html ($attribs)
	{
		return (
			"<input" .
				" name=\"" . $this->name . "\"" .
				" type=\"checkbox\"" .
				" value=\"1\"" .
				(($attribs != "") ? " " . $attribs : "") .
				($this->value ? " checked=\"checked\"" : "") .
			">"
		);
	}
}

class form_field_file extends form_field_parent
{
	protected function _int_get_empty ()
	{
		return (array (
			"name" => "",
			"type" => "",
			"tmp_name" => "",
			"error" => 4,
			"size" => 0
		));
	}

	protected function _int_fetch ($source)
	{
		if (array_key_exists ($this->name, $_FILES) == false) {
			return (null);
		}
		$value = $_FILES [$this->name];
		return ($value);
	}

	static public function get_capabilities ()
	{
		return (array (
			"source" => false,
			"list_name" => false,
			"list_initial" => false
		));
	}

	public function build_html ($attribs)
	{
		return (
			"<input" .
				" name=\"" . $this->name . "\"" .
				" type=\"file\"" .
				(($attribs != "") ? " " . $attribs : "") .
			">"
		);
	}
}

class form_field_hidden extends form_field_parent
{
	protected function _int_get_empty ()
	{
		return ("");
	}

	protected function _int_fetch ($source)
	{
		if (array_key_exists ($this->name, $source) == false) {
			return (null);
		}
		return ($source [$this->name]);
	}

	static public function get_capabilities ()
	{
		return (array (
			"source" => true,
			"list_name" => false,
			"list_initial" => false
		));
	}

	public function build_html ($attribs)
	{
		return (
			"<input" .
				" name=\"" . $this->name . "\"" .
				" type=\"hidden\"" .
				" value=\"" . htmlspecialchars ($this->value) . "\"" .
				(($attribs != "") ? " " . $attribs : "") .
			">"
		);
	}
}

class form_field_image extends form_field_parent
{
	protected function _int_get_empty ()
	{
		return (array ("x" => null, "y" => null));
	}

	protected function _int_fetch ($source)
	{
		$name_x = $this->name . "_x";
		$name_y = $this->name . "_y";
		if (
			(array_key_exists ($name_x, $source) == false) ||
			(array_key_exists ($name_y, $source) == false)
		) {
			return (null);
		}
		return (array ("x" => $source [$name_x], "y" => $source [$name_y]));
	}

	static public function get_capabilities ()
	{
		return (array (
			"source" => true,
			"list_name" => false,
			"list_initial" => false
		));
	}

	public function build_html ($attribs)
	{
		return (
			"<input" .
				" name=\"" . $this->name . "\"" .
				" type=\"image\"" .
				(($attribs != "") ? " " . $attribs : "") .
			">"
		);
	}
}

/*
   FIXME: Must correct the build_html() method

class form_field_multibox extends form_field_parent
{
	protected function _int_get_empty ()
	{
		$result = array ();
		$list = lib_list::get ($this->list_name);
		foreach ($list as $key => $unused) {
			$result [$key] = false;
		}
		return ($result);
	}

	protected function _int_fetch ($source)
	{
		$value = array ();
		$list = lib_list::get ($this->list_name);
		foreach (array_keys ($list) as $key) {
			$value [$key] =
				array_key_exists ($this->name, $source) &&
				is_array ($source [$this->name]) &&
				array_key_exists ($key, $source [$this->name]);
		}
		return ($value);

	}

	static public function get_capabilities ()
	{
		return (array (
			"source" => true,
			"list_name" => true,
			"list_initial" => false
		));
	}

	public function build_html ($current, $columns, $attribs)
	{
		$list = lib_list::get ($this->list_name);
		$keys = array_keys ($list);
		$total = count ($keys);
		$height = ((int) ($total / $columns)) + (($total % $columns) ? 1 : 0);
		$start = ($current - 1) * $height;
		$end = $start + $height;
		$html = "";
		for ($i = $start; $i < $end; ++$i) {
			if ($i < $total) {
				$html .=
					"<input " .
						"name=\"" . $this->name . "[". $keys [$i] ."]\" " .
						"type=\"checkbox\" " .
						"value=\"1\"" .
						(($attribs != "") ? " " . $attribs : "") .
						($this->value [$keys [$i]] ? " checked=\"checked\"" : "") .
					">" .
					$list [$keys [$i]];
			}
			$html .= "<br>\n";
		}
		return ($html);
	}
}
*/

class form_field_password extends form_field_parent
{
	protected function _int_get_empty ()
	{
		return ("");
	}

	protected function _int_fetch ($source)
	{
		if (array_key_exists ($this->name, $source) == false) {
			return (null);
		}
		return ($source [$this->name]);
	}

	static public function get_capabilities ()
	{
		return (array (
			"source" => true,
			"list_name" => false,
			"list_initial" => false
		));
	}

	public function build_html ($attribs)
	{
		return (
			"<input" .
				" name=\"" . $this->name . "\"" .
				" type=\"password\"" .
				" value=\"" . htmlspecialchars ($this->value) . "\"" .
				(($attribs != "") ? " " . $attribs : "") .
			">"
		);
	}
}

class form_field_select extends form_field_parent
{
	protected function _int_get_empty ()
	{
		return ("-1");
	}

	protected function _int_fetch ($source)
	{
		if (array_key_exists ($this->name, $source) == false) {
			return (null);
		}
		return ($source [$this->name]);
	}

	static public function get_capabilities ()
	{
		return (array (
			"source" => true,
			"list_name" => true,
			"list_initial" => true
		));
	}

	public function build_html ($attribs)
	{
		$html =
			"<select " .
				" name=\"" . $this->name . "\"" .
				(($attribs != "") ? " " . $attribs : "") .
			">";
		if ($this->list_initial != null) {
			$html .=
				"<option" .
					" value=\"-1\"" .
					(($this->value == "-1") ? " selected=\"selected\"" : "") .
				">" .
					$this->list_initial .
				"</option>";
		}
		$list = lib_list::get ($this->list_name);
		foreach ($list as $key => $item) {
			$html .=
				"<option" .
					" value=\"" . $key . "\"" .
					(($key == $this->value) ? " selected=\"selected\"" : "") .
				">" .
					$item .
				"</option>";
		}
		$html .= "</select>";
		return ($html);
	}
}

class form_field_text extends form_field_parent
{
	protected function _int_get_empty ()
	{
		return ("");
	}

	protected function _int_fetch ($source)
	{
		if (array_key_exists ($this->name, $source) == false) {
			return (null);
		}
		return ($source [$this->name]);
	}

	static public function get_capabilities ()
	{
		return (array (
			"source" => true,
			"list_name" => false,
			"list_initial" => false
		));
	}

	public function build_html ($attribs)
	{
		return (
			"<input" .
				" name=\"" . $this->name . "\"" .
				" type=\"text\"" .
				" value=\"" . htmlspecialchars ($this->value) . "\"" .
				(($attribs != "") ? " " . $attribs : "") .
			">"
		);
	}
}

class form_field_submit extends form_field_parent
{
	protected function _int_get_empty ()
	{
		return (false);
	}

	protected function _int_fetch ($source)
	{
		return (array_key_exists ($this->name, $source));
	}

	static public function get_capabilities ()
	{
		return (array (
			"source" => true,
			"list_name" => false,
			"list_initial" => false
		));
	}

	public function build_html ($attribs)
	{
		return (
			"<input" .
				" name=\"" . $this->name . "\"" .
				" type=\"submit\"" .
				" value=\"1\"" .
				(($attribs != "") ? " " . $attribs : "") .
			">"
		);
	}
}
?>
