<?php
class sql_engine_postgresql implements sql_engine
{
	private $connect_info;
	private $connection;
	private $query_result;

	public function __construct ($connect_info)
	{
		$this->connect_info = $connect_info;
		$this->connection = null;
		$this->query_result = null;
	}

	public function __destruct ()
	{
		$this->disconnect ();
	}

	static public function is_supported ()
	{
		return (function_exists ("pg_connect"));
	}

	static public function get_capabilities ()
	{
		return (array (
			"oids" => true,
			"plsql" => true,
			"schemas" => true,
			"sequences" => true,
			"transactions" => sql_transaction::SAVEPOINTS,
			"for_share" => "FOR SHARE",
			"for_update" => "FOR UPDATE"
		));
	}

	public function connect ()
	{
		if ($this->connection !== null) {
			return (true);
		}
		$pieces = array ();
		if ($this->connect_info ["username"] !== null) {
			array_push ($pieces, "user='" . $this->escape ($this->connect_info ["username"]) . "'");
		}
		if ($this->connect_info ["password"] !== null) {
			array_push ($pieces, "password='" . $this->escape ($this->connect_info ["password"]) . "'");
		}
		if ($this->connect_info ["connect_1"] !== null) {
			switch ($this->connect_info ["mechanism"]) {
			case sql_mechanism::UNIX:
				$try_hostaddr = false;
				break;
			case sql_mechanism::TCP:
				$try_hostaddr = true;
				break;
			default:
				lib_event::fatal (
					"Unknown connection mechanism \"%s\"",
					$this->connect_info ["mechanism"]
				);
			}
			/*
			   NB: libpq issues DNS lookups even when "host" is a valid IP address,
			   so whenever possible we try to use "hostaddr".
			*/
			array_push (
				$pieces,
				(
					(
						$try_hostaddr &&
						(ip2long ($this->connect_info ["connect_1"]) !== false)
					) ?
					"hostaddr" :
					"host"
				) .
				"=" .
				"'" . $this->escape ($this->connect_info ["connect_1"]) . "'"
			);
		}
		if ($this->connect_info ["connect_2"] !== null) {
			array_push ($pieces, "port='" . $this->escape ($this->connect_info ["connect_2"]) . "'");
		}
		if ($this->connect_info ["database"] !== null) {
			array_push ($pieces, "dbname='" . $this->escape ($this->connect_info ["database"]) . "'");
		}
		if (($connection = pg_connect (implode (" ", $pieces))) === false) {
			return (false);
		}
		$this->connection = $connection;
		return (true);
	}

	public function disconnect ()
	{
		if ($this->connection === null) {
			return (true);
		}
		if (pg_close ($this->connection) == false) {
			return (false);
		}
		$this->connection = null;
		return (true);
	}

	public function escape ($string)
	{
		return (pg_escape_string ($string));
	}

	public function row_to_db ($row, $fields)
	{
		$result = array ();
		reset ($fields);
		foreach ($row as $name => $value_old) {
			$type = each ($fields);
			if ($value_old === null) {
				$value_new = "NULL";
			} else {
				switch ($type ["value"]) {
				case "BOOL":
				case "BOOLEAN":
					$value_new = "'" . ($value_old ? "t" : "f") . "'";
					break;
				case "BYTEA":
					$value_new = "'" . pg_escape_bytea ($value_old) . "'";
					break;
				default:
					$value_new = "'" . pg_escape_string ($value_old) . "'";
					break;
				}
			}
			$result [$name] = $value_new;
		}
		return ($result);
	}

	public function row_from_db ($row, $fields)
	{
		$result = array ();
		reset ($fields);
		foreach ($row as $name => $value_old) {
			$type = each ($fields);
			if ($value_old === null) {
				$value_new = null;
			} else {
				switch ($type ["value"]) {
				case "BOOL":
				case "BOOLEAN":
					$value_new = ($value_old == "t");
					break;
				case "BYTEA":
					$value_new = pg_unescape_bytea ($value_old);
					break;
				default:
					$value_new = $value_old;
					break;
				}
			}
			$result [$name] = $value_new;
		}
		return ($result);
	}

	public function query ($request)
	{
		if ($this->connect () == false) {
			return (false);
		}
		if (($this->query_result = pg_query (
			$this->connection,
			$request
		)) === false) {
			return (false);
		}
		return (true);
	}

	public function error ()
	{
		return (pg_last_error ($this->connection));
	}

	public function num_fields ()
	{
		if (($num = pg_num_fields ($this->query_result)) == -1) {
			return (false);
		}
		return ($num);
	}

	public function fetch ()
	{
		return (pg_fetch_assoc ($this->query_result));
	}

	public function affected_rows ()
	{
		return (pg_affected_rows ($this->query_result));
	}
}
?>
