<?php
interface sql_engine
{
	const MYSQL = 1;
	const POSTGRESQL = 2;
	const SQLITE = 3;

	/*
		NB:
		1. The methods should not return any database
		resources or objects because disconnecting for
		PDO drivers is performed by destructing the
		the PDO object and dangling external references
		to PDO objects may prevent the reference count
		from going to zero.

		2. The connect and disconnect methods are public
		just for convenience since the sql_engine_XXXX
		objects connect to and disconnect from the
		database automatically. However the connect()
		method is convenient if you want to explicitly
		check if your database connect info is correct.
	*/
	static public function is_supported ();
	static public function get_capabilities ();
	public function connect ();
	public function disconnect ();
	public function escape ($string);
	public function row_to_db ($row, $fields);
	public function row_from_db ($row, $fields);
	public function query ($request);
	public function error ();
	public function num_fields ();
	public function fetch ();
	public function affected_rows ();
}

interface sql_mechanism
{
	const UNIX = 1;
	const TCP = 2;
}

class sql_manager
{
	private $engine;
	private $caps;
	private $transaction;

	public function __construct ($connect_info)
	{
		$this->engine = self::create_engine ($connect_info);
		$this->caps = $this->engine->get_capabilities ();
		$this->transaction = new sql_transaction ($this->engine);
	}

	public function __destruct ()
	{
		$this->transaction->shutdown ();
	}

	static public function create_engine ($connect_info)
	{
		switch ($connect_info ["engine"]) {
		case sql_engine::MYSQL:
			return (new sql_engine_mysql ($connect_info));
		case sql_engine::POSTGRESQL:
			return (new sql_engine_postgresql ($connect_info));
		case sql_engine::SQLITE:
			return (new sql_engine_sqlite ($connect_info));
		default:
			lib_event::fatal (
				"Attempt to create an unknown database engine type \"%s\"",
				$connect_info ["engine"]
			);
		}
	}

	public function get_capabilities ()
	{
		return ($this->caps);
	}

	public function escape ($string)
	{
		return ($this->engine->escape ($string));
	}

	public function affected_rows ()
	{
		return ($this->engine->affected_rows ());
	}

	public function row_to_db ($row, $fields)
	{
		return ($this->engine->row_to_db ($row, $fields));
	}

	public function query ($request, $fields)
	{
		lib_event::log (
			lib_event::L_DEBUG,
			"SQL query: \"%s\", fields map: %s",
			$request,
			($fields !== null) ? "YES" : "NO"
		);
		if ($this->transaction->is_active () == false) {
			switch ($this->caps ["transactions"]) {
			case sql_transaction::NONE:
				break;
			case sql_transaction::BASIC:
			case sql_transaction::SAVEPOINTS:
				$this->transaction->start ();
				break;
			default:
				lib_event::fatal (
					"Unknown transaction support value \"%s\"",
					$this->caps ["transactions"]
				);
			}
		}
		if ($this->engine->query ($request) === false) {
			$error = $this->engine->error ();
			if ($this->transaction->is_active ()) {
				$this->transaction->rollback ();
			}
			lib_event::fatal (
				"SQL query \"%s\" failed with \"%s\"",
				$request,
				$error
			);
		}
		if ($fields !== null) {
			if (($num_fields = $this->engine->num_fields ()) === false) {
				if ($this->transaction->is_active ()) {
					$this->transaction->rollback ();
				}
				lib_event::fatal ("Failed to get number of result fields");
			}
			if ($num_fields != count ($fields)) {
				if ($this->transaction->is_active ()) {
					$this->transaction->rollback ();
				}
				lib_event::fatal (
					"Number of result fields (%u) differs from the number of map fields (%u)",
					$num_fields,
					count ($fields)
				);
			}
		}
		$rows = array ();
		while (($one_row = $this->engine->fetch ()) !== false) {
			if ($fields !== null) {
				$one_row = $this->engine->row_from_db (
					$one_row,
					$fields
				);
			}
			array_push ($rows, $one_row);
		}
		return ($rows);
	}

	public function commit ()
	{
		if ($this->transaction->is_active ()) {
			$this->transaction->commit ();
		}
	}

	public function savepoint_create ()
	{
		if ($this->transaction->is_active () == false) {
			$this->transaction->start ();
		}
		return ($this->transaction->savepoint_create ());
	}

	public function savepoint_rollback ($index)
	{
		if ($this->transaction->is_active () == false) {
			$this->transaction->start ();
		}
		$this->transaction->savepoint_rollback ($index);
	}

	public function savepoint_release ($index)
	{
		if ($this->transaction->is_active () == false) {
			$this->transaction->start ();
		}
		$this->transaction->savepoint_release ($index);
	}
}
?>
