<?php
/*
	NB: Keeping the SQL manager and the transaction object separate allows us to
	shutdown the SQL manager just be removing all references to the manager object.
	(see the note for the shutdown method).
*/
class sql_transaction
{
	const NONE = 1;
	const BASIC = 2;
	const SAVEPOINTS = 3;

	private $engine;
	private $support;
	private $cookie;
	private $savepoint;

	public function __construct ($engine)
	{
		$this->engine = $engine;
		$caps = $this->engine->get_capabilities ();
		$this->support = $caps ["transactions"];
		$this->cookie = null;
		$this->savepoint = 0;
	}

	public function __destruct ()
	{
		$this->shutdown ();
	}

	/*
		NB: When there is an open transaction, the lib_shutdown_callbacks keeps an
		external reference to the transaction object, so it is not possible to
		free the transaction object by removing all external references to it.
		That is why the shutdown method must be called explicitly in order to
		remove the external reference to the transaction object.
	*/
	public function shutdown ()
	{
		if ($this->is_active ()) {
			$this->rollback ();
		}
	}

	public function is_active ()
	{
		return ($this->cookie !== null);
	}

	public function start ()
	{
		if ($this->is_active ()) {
			lib_event::fatal ("Cannot start a transaction - already active");
		}
		switch ($this->support) {
		case self::NONE:
			lib_event::fatal ("The database driver does not support transactions");
		case self::BASIC:
		case self::SAVEPOINTS:
			break;
		default:
			lib_event::fatal ("Unknown transaction support value \"%s\"", $this->support);
		}
		if ($this->engine->query ("START TRANSACTION") === false) {
			lib_event::fatal (
				"SQL transaction start failed with \"%s\"",
				$this->engine->error ()
			);
		}
		$this->cookie = lib_shutdown_callbacks::add (array (
			$this,
			"rollback"
		));
	}

	public function commit ()
	{
		if ($this->is_active () == false) {
			lib_event::fatal ("Cannot commit a transaction - no active transaction");
		}
		if ($this->engine->query ("COMMIT") === false) {
			lib_event::fatal (
				"SQL transaction commit failed with \"%s\"",
				$this->engine->error ()
			);
		}
		lib_shutdown_callbacks::remove ($this->cookie);
		$this->cookie = null;
	}

	public function rollback ()
	{
		if ($this->is_active () == false) {
			lib_event::fatal ("Cannot rollback a transaction - no active transaction");
		}
		if ($this->engine->query ("ROLLBACK") === false) {
			lib_event::fatal (
				"SQL transaction rollback failed with \"%s\"",
				$this->engine->error ()
			);
		}
		lib_shutdown_callbacks::remove ($this->cookie);
		$this->cookie = null;
	}

	public function savepoint_create ()
	{
		if ($this->is_active () == false) {
			lib_event::fatal ("Cannot create a savepoint - no active transaction");
		}
		$index = $this->savepoint++;
		if ($this->engine->query ("SAVEPOINT sp_" . $index) === false) {
			lib_event::fatal (
				"Failed to create savepoint %s, error \"%s\"",
				$index,
				$this->engine->error ()
			);
		}
		return ($index);
	}

	public function savepoint_rollback ($index)
	{
		if ($this->is_active () == false) {
			lib_event::fatal ("Cannot rollback to a savepoint - no active transaction");
		}
		if ($this->engine->query ("ROLLBACK TO SAVEPOINT sp_" . $index) === false) {
			lib_event::fatal (
				"Failed to rollback to savepoint %s, error \"%s\"",
				$index,
				$this->engine->error ()
			);
		}
	}

	public function savepoint_release ($index)
	{
		if ($this->is_active () == false) {
			lib_event::fatal ("Cannot release a savepoint - no active transaction");
		}
		if ($this->engine->query ("RELEASE SAVEPOINT sp_" . $index) === false) {
			lib_event::fatal (
				"Failed to release savepoint %s, error \"%s\"",
				$index,
				$this->engine->error ()
			);
		}
	}
}
?>
