<?php
class lib_db
{
	const DB_PIECE_TEXT = 1;
	const DB_PIECE_PARAM = 2;
	const DB_PIECE_RAW = 3;

	static private $manager;
	static private $caps;

	static private $sequences;
	static private $queries;
	static private $tables;
	static private $views;

	/* NB: This should really be a static constructor */
	static public function initialize ()
	{
		self::$manager = new sql_manager (array (
			"engine" => DB_ENGINE,
			"mechanism" => DB_MECHANISM,
			"connect_1" => DB_CONNECT_1,
			"connect_2" => DB_CONNECT_2,
			"username" => DB_USERNAME,
			"password" => DB_PASSWORD,
			"database" => DB_DATABASE
		));
		self::$caps = self::$manager->get_capabilities ();
		require (SYSTEM_DIR . "include/generated/database.inc");
	}

	static public function get_capabilities ()
	{
		return (self::$caps);
	}

	static public function next_id ($sequence)
	{
		if (array_key_exists ($sequence, self::$sequences) == false) {
			lib_event::fatal ("Attempt to read a non-existing sequence \"%s\"", $sequence);
		}
		if (self::$caps ["sequences"]) {
			$rows = self::$manager->query (
				"SELECT nextval(" . self::_int_sql_value ($sequence) . ") AS nextval",
				null
			);
			$result = $rows [0]["nextval"];
		} else {
			$rows = self::$manager->query (
				"SELECT value " .
				"FROM _fwk_sequences " .
				"WHERE name=" . self::_int_sql_value ($sequence),
				null
			);
			$result = $rows [0]["value"];
			self::$manager->query (
				"UPDATE _fwk_sequences " .
				"SET value=value+1 " .
				"WHERE name=" . self::_int_sql_value ($sequence),
				null
			);
		}
		return ($result);
	}

	static public function escape ($string)
	{
		return (self::$manager->escape ($string));
	}

	static public function query_raw ($query)
	{
		return (self::$manager->query ($query, null));
	}

	static public function query_named ($name)
	{
		if (array_key_exists ($name, self::$queries) == false) {
			lib_event::fatal ("Attempt to execute an unknown named query \"%s\"", $name);
		}
		$query = "";
		foreach (self::$queries [$name]["pieces"] as $one_piece) {
			switch ($one_piece ["type"]) {
			case self::DB_PIECE_TEXT:
				$query .= $one_piece ["value"];
				break;
			case self::DB_PIECE_PARAM:
				$query .= self::$manager->escape (func_get_arg ($one_piece ["value"] + 1));
				break;
			case self::DB_PIECE_RAW:
				$query .= func_get_arg ($one_piece ["value"] + 1);
				break;
			default:
				lib_event::fatal ("Unknown type of query piece \"%s\"", $one_piece ["type"]);
			}
		}
		return (self::$manager->query (
			$query,
			self::$queries [$name]["fields"]
		));
	}

	static public function affected_rows ()
	{
		return (self::$manager->affected_rows ());
	}

	static public function create ($table, $row)
	{
		if (array_key_exists ($table, self::$tables) == false) {
			lib_event::fatal (
				"Attempt to create a database object in a non-existent table \"%s\"",
				$table
			);
		}
		self::_int_sql_insert ($table, $row);
	}

	static public function destroy ($table, $primary)
	{
		if (array_key_exists ($table, self::$tables) == false) {
			lib_event::fatal (
				"Attempt to destroy a database object in a non-existent table \"%s\"",
				$table
			);
		}
		self::_int_sql_delete (
			$table,
			array (self::$tables [$table]["primary"] => $primary)
		);
	}

	static public function destroy_by_filter ($table, $filter)
	{
		if (array_key_exists ($table, self::$tables) == false) {
			lib_event::fatal (
				"Attempt to destroy a database object in a non-existent table \"%s\"",
				$table
			);
		}
		self::_int_sql_delete ($table, $filter);
	}

	static public function read ($table, $primary)
	{
		if (array_key_exists ($table, self::$tables) == false) {
			lib_event::fatal (
				"Attempt to read a database object from a non-existent table \"%s\"",
				$table
			);
		}
		return (self::_int_sql_select (
			$table,
			array (self::$tables [$table]["primary"] => $primary)
		));
	}

	static public function read_by_filter ($table, $filter)
	{
		if (array_key_exists ($table, self::$tables) == false) {
			lib_event::fatal (
				"Attempt to read a database object from a non-existent table \"%s\"",
				$table
			);
		}
		return (self::_int_sql_select ($table, $filter));
	}

	static public function write ($table, $row, $primary)
	{
		if (array_key_exists ($table, self::$tables) == false) {
			lib_event::fatal (
				"Attempt to write a database object to a non-existent table \"%s\"",
				$table
			);
		}
		self::_int_sql_update (
			$table,
			$row,
			array (self::$tables [$table]["primary"] => $primary)
		);
	}

	static public function write_by_filter ($table, $row, $filter)
	{
		if (array_key_exists ($table, self::$tables) == false) {
			lib_event::fatal (
				"Attempt to write a database object to a non-existent table \"%s\"",
				$table
			);
		}
		self::_int_sql_update ($table, $row, $filter);
	}

	static public function build_filter ($table, $values)
	{
		if (array_key_exists ($table, self::$tables) == false) {
			lib_event::fatal (
				"Attempt to build a filter string for a non-existent table \"%s\"",
				$table
			);
		}
		return (self::_int_filter_string (
			$values,
			self::$tables [$table]["fields"],
			false
		));

	}

	static public function commit ()
	{
		self::$manager->commit ();
	}

	static public function savepoint_create ()
	{
		return (self::$manager->savepoint_create ());
	}

	static public function savepoint_rollback ($index)
	{
		self::$manager->savepoint_rollback ($index);
	}

	static public function savepoint_release ($index)
	{
		self::$manager->savepoint_release ($index);
	}

	static private function _int_sql_select ($table, $filter)
	{
		$fields = self::$tables [$table]["fields"];
		$rows = self::$manager->query (
			"SELECT * " .
			"FROM " . self::$manager->escape ($table) .
			self::_int_filter_string ($filter, $fields, true),
			$fields
		);
		switch (count ($rows)) {
		case 0:
			return (null);
		case 1:
			break;
		default:
			lib_event::fatal ("Multiple rows received when reading from a table");
		}
		return ($rows [0]);
	}

	static private function _int_sql_insert ($table, $row)
	{
		$items = self::_int_sql_row (
			$row,
			self::$tables [$table]["fields"]
		);
		self::$manager->query (
			"INSERT INTO " . self::$manager->escape ($table) . " (" .
				implode (", ", array_keys ($items)) .
			") VALUES (" .
				implode (", ", array_values ($items)) .
			")",
			null
		);
	}

	static private function _int_sql_update ($table, $row, $filter)
	{
		if ($row == array ()) {
			return;
		}
		$fields = self::$tables [$table]["fields"];
		$list = array ();
		$items = self::_int_sql_row ($row, $fields);
		foreach ($items as $key => $value) {
			array_push ($list, $key . "=" . $value);
		}
		self::$manager->query (
			"UPDATE " . self::$manager->escape ($table) . " " .
			"SET " . implode (", ", $list) .
			self::_int_filter_string ($filter, $fields, true),
			null
		);
	}

	static private function _int_sql_delete ($table, $filter)
	{
		self::$manager->query (
			"DELETE FROM " . self::$manager->escape ($table) .
			self::_int_filter_string (
				$filter,
				self::$tables [$table]["fields"],
				true
			),
			null
		);
	}

	static private function _int_filter_string ($filter, $fields, $add_where)
	{
		if (count ($filter) == 0) {
			return ("");
		}
		$list = array ();
		$items = self::_int_sql_row ($filter, $fields);
		foreach ($items as $key => $value) {
			array_push ($list, $key . "=" . $value);
		}
		return (
			($add_where ? " WHERE " : "") .
			implode (" AND ", $list)
		);
	}

	static private function _int_sql_value ($value)
	{
		return (
			is_null ($value) ?
			"NULL" :
			"'" . self::$manager->escape ($value) . "'"
		);
	}

	static private function _int_sql_row ($row, $fields)
	{
		/*
			NB: In PHP 5.1+ this can be replaced with:
			$types = array_values (array_intersect_key ($fields, $row));
		*/
		$types = array ();
		foreach (array_keys ($row) as $name) {
			array_push ($types, $fields [$name]);
		}
		return (
			count ($row) ?
			array_combine (
				array_map (
					array (self::$manager, "escape"),
					array_keys ($row)
				),
				self::$manager->row_to_db ($row, $types)
			) :
			array ()
		);
	}
}
lib_db::initialize ();
?>
