<?php
class lib_file
{
	static private $is_windows;

	/* NB: This should really be a static constructor */
	static public function initialize ()
	{
		self::$is_windows = lib_misc::is_windows ();
	}

	static public function path_split ($path)
	{
		if (self::$is_windows && preg_match ('/^([A-Za-z]):(.*)$/', $path, $pieces)) {
			$drive = $pieces [1];
			$path = $pieces [2];
		} else {
			$drive = null;
		}
		if (preg_match ('/^(?:\\/|\\\\)+(.*)$/', $path, $pieces)) {
			$absolute = true;
			$path = $pieces [1];
		} else {
			$absolute = false;
		}
		$parts = preg_split ('/(\\/|\\\\)+/', $path);
		if (end ($parts) == "") {
			array_pop ($parts);
		}
		return (array (
			"drive" => $drive,
			"absolute" => $absolute,
			"parts" => $parts
		));
	}

	static public function path_join ($info, $separator)
	{
		$path = "";
		if (self::$is_windows && ($info ["drive"] != null)) {
			$path .= $info ["drive"] . ":";
		}
		if ($info ["absolute"]) {
			$path .= $separator;
		}
		$path .= implode ($separator, $info ["parts"]);
		return ($path);
	}

	static public function path_is_absolute ($path)
	{
		$info = self::path_split ($path);
		return ($info ["absolute"]);
	}

	static public function path_dirname ($path)
	{
		$info = self::path_split ($path);
		if (array_pop ($info ["parts"]) === null) {
			return (false);
		}
		return (self::path_join ($info, DIRECTORY_SEPARATOR));
	}

	static public function path_normalize ($path)
	{
		if (($info = self::_int_normalize_info (
			self::path_split ($path)
		)) === false) {
			return (false);
		}
		return (self::path_join ($info, DIRECTORY_SEPARATOR));
	}

	static private function _int_normalize_info ($info)
	{
		$parts = array ();
		foreach ($info ["parts"] as $item) {
			switch ($item) {
			case ".":
				break;
			case "..":
				$last = end ($parts);
				if (($last !== false) && ($last !== "..")) {
					array_pop ($parts);
				} else {
					if ($info ["absolute"]) {
						return (false);
					} else {
						array_push ($parts, $item);
					}
				}
				break;
			default:
				if (self::$is_windows) {
					$item = strtolower ($item);
				}
				array_push ($parts, $item);
			}
		}
		return (array (
			"drive" => $info ["drive"],
			"absolute" => $info ["absolute"],
			"parts" => $parts
		));
	}

	static public function path_unixize ($path)
	{
		return (self::path_join (self::path_split ($path), "/"));
	}

	static public function path_relative ($source, $dest)
	{
		$info_source = self::path_split ($source);
		$info_dest = self::path_split ($dest);
		if (self::$is_windows && ($info_source ["drive"] != $info_dest ["drive"])) {
			return (false);
		}
		if (($info_source ["absolute"] == false) || ($info_dest ["absolute"] == false)) {
			return (false);
		}
		if (($info_source = self::_int_normalize_info ($info_source)) === false) {
			return (false);
		}
		if (($info_dest = self::_int_normalize_info ($info_dest)) === false) {
			return (false);
		}
		$parts_source = $info_source ["parts"];
		$parts_dest = $info_dest ["parts"];
		$parts_result = array ();
		while (count ($parts_source) && count ($parts_dest) && ($parts_source [0] == $parts_dest [0])) {
			array_shift ($parts_source);
			array_shift ($parts_dest);
		}
		$parts_result = array_pad ($parts_result, count ($parts_source), "..");
		$parts_result = array_merge ($parts_result, $parts_dest);
		return (self::path_join (
			array (
				"drive" => $info_source ["drive"],
				"absolute" => false,
				"parts" => $parts_result
			),
			DIRECTORY_SEPARATOR
		));
	}

	/*
	   Properties in the $params parameter:
		keep_ownership		If true then will keep ownership of copied files.
		keep_permissions	If true then will keep filesystem permissions of copied files.
	*/
	static public function tree_copy ($source, $dest, $params)
	{
		if (($norm_source = self::path_normalize ($source)) === false) {
			return (false);
		}
		if (($norm_dest = self::path_normalize ($dest)) === false) {
			return (false);
		}
		/*
		    - We don't want to follow symlinks, since following them will make us, either copy files that do not belong
		      to the source tree, or copy some part of the source tree twice.
		    - We don't want to mark directories with a trailing slash since some of them may be symlinks that should be
		      created via symlink().
		    - We want partial path during walk, so we can easily build the destination filenames by simply appending the
		      walked filenames to the destination.
		*/
		if (($source_tree = self::tree_walk ($norm_source, array (
			"dir_follow" => false,
			"dir_mark" => false,
			"path_partial" => true,
			"path_detailed" => false,
			"depth_min" => null,
			"depth_max" => null
		))) == false) {
			return (false);
		}
		if (@lstat ($norm_dest)) {
			if (self::tree_delete ($norm_dest) == false) {
				return (false);
			}
		}
		foreach ($source_tree as $one_file) {
			if (self::_int_tree_copy_file (
				$norm_source,
				$norm_dest,
				$one_file,
				$params
			) == false) {
				return (false);
			}
		}
		return (true);
	}

	static private function _int_tree_copy_file ($source, $dest, $item, $params)
	{
		if ($item ["pathname"] != "") {
			$source .= DIRECTORY_SEPARATOR . $item ["pathname"];
			$dest .= DIRECTORY_SEPARATOR . $item ["pathname"];
		}
		if ($item ["is_link"]) {
			if (($target = @readlink ($source)) == false) {
				return (false);
			}
			if (@symlink ($target, $dest) == false) {
				return (false);
			}
		} else if ($item ["is_dir"]) {
			if (@mkdir ($dest, 0755) == false) {
				return (false);
			}
		} else {
			if (copy ($source, $dest) == false) {
				return (false);
			}
		}
		if ($item ["is_link"] == false) {
			if ($params ["keep_ownership"]) {
				if (@chown ($dest, $item ["uid"]) == false) {
					return (false);
				}
				if (@chgrp ($dest, $item ["gid"]) == false) {
					return (false);
				}
			}
			if ($params ["keep_permissions"]) {
				if (@chmod ($dest, $item ["permissions"]) == false) {
					return (false);
				}
			}
		}
		return (true);
	}

	static public function tree_delete ($pathname)
	{
		/*
		    - We don't want to follow symlinks, since following them will destroy a filetree that does not belong to us
		      or try to destroy a part of our filetree twice.
		    - We don't want to mark directories with a trailing slash, since an attempt to unlink "/path/to/some/symlink/"
		      will cause unlink() to return a "Not a directory" error.
		*/
		if (($tree = self::tree_walk ($pathname, array (
			"dir_follow" => false,
			"dir_mark" => false,
			"path_partial" => false,
			"path_detailed" => false,
			"depth_min" => null,
			"depth_max" => null
		))) == false) {
			return (false);
		}
		while (($one_file = array_pop ($tree)) !== null) {
			if ($one_file ["is_dir"] && ($one_file ["is_link"] == false)) {
				if (@rmdir ($one_file ["pathname"]) == false) {
					return (false);
				}
			} else {
				if (@unlink ($one_file ["pathname"]) == false) {
					return (false);
				}
			}
		}
		return (true);
	}

	static public function tree_move ($source, $dest)
	{
		if (($norm_source = self::path_normalize ($source)) === false) {
			return (false);
		}
		if (($norm_dest = self::path_normalize ($dest)) === false) {
			return (false);
		}
		if (@lstat ($norm_dest)) {
			if (self::tree_delete ($norm_dest) == false) {
				return (false);
			}
		}
		/*
			NB: If rename() fails, we assume an EXDEV error code. This is far
			from perfect, but at the moment PHP does not allow us to get the
			exact error code.
		*/
		if (@rename ($norm_source, $norm_dest) == false) {
			/*
				NB: As of PHP 5.1.4, if you try to rename a directory and it
				fails with EXDEV, it will leave a zero-sized regular file in
				place of the destination.
			*/
			if (self::tree_copy (
				$norm_source,
				$norm_dest,
				array (
					"keep_ownership" => true,
					"keep_permissions" => true
				)
			) == false) {
				return (false);
			}
			if (self::tree_delete ($norm_source) == false) {
				return (false);
			}
		}
		return (true);
	}

	/*
	   Properties in the $params parameter:
		dir_follow	If true then will follow symlinks.
		dir_mark	If true then will add trailing directory separator to each directory.
		path_partial	If true then partial pathnames are added to the list.
		path_detailed	If true then detailed info arrays are added to the list instead of string pathnames.
		depth_min	If non-null then specifies minimum allowed depth of added directories.
		depth_max	If non-null then specifies maximum allowed depth of added directories.
	*/
	static public function tree_walk ($pathname, $params)
	{
		if (($norm_pathname = self::path_normalize ($pathname)) === false) {
			return (false);
		}
		if (($tree = self::_int_tree_get ($norm_pathname, $params, 0)) === false) {
			return (false);
		}
		return (self::_int_tree_merge ($norm_pathname, $tree, $params));
	}

	static private function _int_tree_get ($pathname, $params, $depth)
	{
		if (($params ["depth_max"] !== null) && ($depth > $params ["depth_max"])) {
			return (array ());
		}
		if (@lstat ($pathname) == false) {
			return (false);
		}
		if (($stats = @stat ($pathname)) == false) {
			$info = array (
				"pathname" => array (),
				"is_dir" => null,
				"is_executable" => null,
				"is_file" => null,
				"is_link" => is_link ($pathname),
				"is_readable" => null,
				"is_writable" => null,
				"uid" => null,
				"gid" => null,
				"permissions" => null
			);
		} else {
			$info = array (
				"pathname" => array (),
				"is_dir" => is_dir ($pathname),
				"is_executable" => is_executable ($pathname),
				"is_file" => is_file ($pathname),
				"is_link" => is_link ($pathname),
				"is_readable" => is_readable ($pathname),
				"is_writable" => is_writable ($pathname),
				"uid" => $stats ["uid"],
				"gid" => $stats ["gid"],
				"permissions" => $stats ["mode"]
			);
		}
		$result =
			(($params ["depth_min"] === null) || ($depth >= $params ["depth_min"])) ?
			array ($info) :
			array ();
		if ($info ["is_dir"] && ($params ["dir_follow"] || ($info ["is_link"] == false))) {
			if (($files = scandir ($pathname)) == false) {
				return (false);
			}
			++$depth;
			foreach ($files as $one_file) {
				if (($one_file != ".") && ($one_file != "..")) {
					if (($items = self::_int_tree_get (
						$pathname . DIRECTORY_SEPARATOR .  $one_file,
						$params,
						$depth
					)) === false) {
						return (false);
					}
					foreach ($items as &$one_item) {
						array_unshift ($one_item ["pathname"], $one_file);
					}
					$result = array_merge ($result, $items);
				}
			}
		}
		return ($result);
	}

	static private function _int_tree_merge ($pathname, $tree, $params)
	{
		$prefix = self::path_split ($pathname);
		foreach ($tree as &$one_file) {
			$details =
				$params ["path_partial"] ?
				array (
					"drive" => null,
					"absolute" => false,
					"parts" => $one_file ["pathname"]
				) :
				array (
					"drive" => $prefix ["drive"],
					"absolute" => $prefix ["absolute"],
					"parts" => array_merge ($prefix ["parts"], $one_file ["pathname"])
				);
			if ($params ["path_detailed"]) {
				$one_file ["pathname"] = $details;
			} else {
				$one_file ["pathname"] = self::path_join (
					$details,
					DIRECTORY_SEPARATOR
				);
				if ($params ["dir_mark"] && $one_file ["is_dir"]) {
					$one_file ["pathname"] .= DIRECTORY_SEPARATOR;
				}
			}
		}
		return ($tree);
	}

	static public function create_temp_file ($dir)
	{
		if (($norm_dir = self::path_normalize ($dir)) === false) {
			return (false);
		}
		for (;;) {
			$pathname =
				$norm_dir .
				(($norm_dir != "") ? DIRECTORY_SEPARATOR : "") .
				lib_misc::generate_hash (32);
			if (($handle = @fopen ($pathname, "x")) !== false) {
				if (@fclose ($handle) == false) {
					return (false);
				}
				break;
			} else {
				if (@lstat ($pathname) == false) {
					return (false);
				}
			}
		}
		return ($pathname);
	}

	static public function create_temp_dir ($dir)
	{
		if (($norm_dir = self::path_normalize ($dir)) === false) {
			return (false);
		}
		for (;;) {
			$pathname =
				$norm_dir .
				(($norm_dir != "") ? DIRECTORY_SEPARATOR : "") .
				lib_misc::generate_hash (32);
			if (@mkdir ($pathname, 0755)) {
				break;
			} else {
				if (@lstat ($pathname) == false) {
					return (false);
				}
			}
		}
		return ($pathname);
	}
}
lib_file::initialize ();
?>
